﻿Imports System.Drawing.Drawing2D

Public Class Form1
    ' The points selected by the user.
    Private ShapePoints As New List(Of Point)()
    Private ShapePath As GraphicsPath = Nothing
    Private LinesPath As GraphicsPath = Nothing
    Private IsDrawing As Boolean = False

    ' Start drawing.
    Private Sub Form1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        ShapePoints = New List(Of Point)()
        ShapePoints.Add(e.Location)
        IsDrawing = True
        Refresh()
    End Sub

    ' Continue drawing.
    Private Sub Form1_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        If Not IsDrawing Then Return
        ShapePoints.Add(e.Location)
        Refresh()
    End Sub

    ' Finish drawing.
    Private Sub Form1_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        If Not IsDrawing Then Return
        IsDrawing = False

        ' Generate the random lines to fill the shape.
        GenerateLines()

        Refresh()
    End Sub

    ' Generate the random lines to fill the shape.
    Private Sub GenerateLines()
        If ShapePoints.Count < 3 Then
            ShapePath = Nothing
            LinesPath = Nothing
            Return
        End If

        ' Make the shape's path.
        ShapePath = New GraphicsPath()
        ShapePath.AddPolygon(ShapePoints.ToArray())

        ' Get the shape's bounds.
        Dim bounds As RectangleF = ShapePath.GetBounds()
        Dim xmin As Integer = CInt(bounds.Left)
        Dim xmax As Integer = CInt(bounds.Right) + 1
        Dim ymin As Integer = CInt(bounds.Top)
        Dim ymax As Integer = CInt(bounds.Bottom) + 1

        ' Generate random lines.
        LinesPath = New GraphicsPath()
        Dim num_lines As Integer = CInt((bounds.Width + bounds.Height) / 16)
        Dim rand As New Random()
        Dim x1, y1, x2, y2 As Integer
        For i As Integer = 1 To num_lines
            x1 = rand.Next(xmin, xmax)
            y1 = ymin
            x2 = rand.Next(xmin, xmax)
            y2 = ymax
            LinesPath.AddLine(x1, y1, x2, y2)
            x1 = xmin
            y1 = rand.Next(ymin, ymax)
            x2 = xmax
            y2 = rand.Next(ymin, ymax)
            LinesPath.AddLine(x1, y1, x2, y2)
        Next i
    End Sub

    ' Draw the shape.
    Private Sub Form1_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MyBase.Paint
        ' Draw the shape.
        If IsDrawing Then
            ' Draw the lines so far.
            If ShapePoints.Count > 1 Then
                e.Graphics.DrawLines(Pens.Green, ShapePoints.ToArray())
            End If
        Else
            ' Fill and outline the finished shape.
            If ShapePath IsNot Nothing Then
                e.Graphics.FillPath(Brushes.LightGreen, ShapePath)
                e.Graphics.DrawPath(Pens.Green, ShapePath)

                ' Fill with the lines.
                e.Graphics.Clip = New Region(ShapePath)
                e.Graphics.DrawPath(Pens.Green, LinesPath)
            End If
        End If
    End Sub
End Class
